package com.mc.monkeynes.gui;

import com.mc.monkeynes.base.*;
import com.mc.monkeynes.hardware.*;
import com.mc.monkeynes.hardware.mapper.*;

import java.awt.*;
import java.awt.event.*;
//import java.util.*;
import javax.swing.*;
import javax.swing.event.*;



public class MonkeyNesGui
{
  private JFrame             mainFrame                        = new JFrame();
  private JFileChooser       fileChooser                      = null;
  private GenericFileFilter  romTypeFilter                    = new GenericFileFilter( "nes", "NES ROM Files (*.nes)", true );

  // Menu System
  private JMenuBar           mainMenu                         = new JMenuBar();
  private JMenu              fileMenu                         = new JMenu( "File" );
  private JMenuItem          fileOpenMenuItem                 = new JMenuItem( "Open ROM", KeyEvent.VK_O );
  private JMenuItem          fileCloseMenuItem                = new JMenuItem( "Close ROM", KeyEvent.VK_C );
  private JMenuItem          fileExitMenuItem                 = new JMenuItem( "Exit", KeyEvent.VK_X );
  private JMenu              cpuMenu                          = new JMenu( "CPU" );
  private JCheckBoxMenuItem  cpuPowerMenuItem                 = new JCheckBoxMenuItem( "Power", false );
  private JMenuItem          cpuResetMenuItem                 = new JMenuItem( "Reset", KeyEvent.VK_R );
  private JMenu              optMenu                          = new JMenu( "Options" );
  private JMenuItem          optCpuMenuItem                   = new JMenuItem( "CPU...", KeyEvent.VK_C );
  private JMenuItem          optVideoMenuItem                 = new JMenuItem( "Video...", KeyEvent.VK_V );
  private JMenu              optSoundMenu                     = new JMenu( "Sound" );
  private JMenuItem          optInputMenuItem                 = new JMenuItem( "Input...", KeyEvent.VK_I );
  private JCheckBoxMenuItem  optSndSquare1MenuItem            = new JCheckBoxMenuItem( "Square 1", true );
  private JCheckBoxMenuItem  optSndSquare2MenuItem            = new JCheckBoxMenuItem( "Square 2", true );
  private JCheckBoxMenuItem  optSndTriangleMenuItem           = new JCheckBoxMenuItem( "Triangle", true );
  private JCheckBoxMenuItem  optSndNoiseMenuItem              = new JCheckBoxMenuItem( "Noise", true );
  private JCheckBoxMenuItem  optSndPcmMenuItem                = new JCheckBoxMenuItem( "PCM", true );
  private JMenu              toolMenu                         = new JMenu( "Tools" );
  private JMenuItem          toolPatternViewerMenuItem        = new JMenuItem( "Pattern Table Viewer" );
  private JMenu              helpMenu                         = new JMenu( "Help" );
  private JMenuItem          helpAboutMenuItem                = new JMenuItem( "About", KeyEvent.VK_A );

  // Status label
  private JLabel             statusLabel                      = new JLabel( "Idle" );

  // Other stuff
  private NesRom             rom                              = null;
  private boolean            emulating                        = false;

  public MonkeyNesGui()
  {
    // Put the menu system together -------------
    // File
    fileMenu.setMnemonic( KeyEvent.VK_F );
    fileMenu.add( fileOpenMenuItem );
    fileMenu.add( fileCloseMenuItem );
    fileMenu.add( fileExitMenuItem );

    // CPU
    cpuMenu.setMnemonic( KeyEvent.VK_C );
    cpuMenu.add( cpuPowerMenuItem );
    cpuMenu.add( cpuResetMenuItem );

    // Options
    optMenu.setMnemonic( KeyEvent.VK_O );
    optSoundMenu.setMnemonic( KeyEvent.VK_S );
    optSndSquare1MenuItem.setMnemonic( KeyEvent.VK_1 );
    optSndSquare2MenuItem.setMnemonic( KeyEvent.VK_2 );
    optSndTriangleMenuItem.setMnemonic( KeyEvent.VK_T );
    optSndNoiseMenuItem.setMnemonic( KeyEvent.VK_N );
    optSndPcmMenuItem.setMnemonic( KeyEvent.VK_P );
    optSoundMenu.add( optSndSquare1MenuItem );
    optSoundMenu.add( optSndSquare2MenuItem );
    optSoundMenu.add( optSndTriangleMenuItem );
    optSoundMenu.add( optSndNoiseMenuItem );
    optSoundMenu.add( optSndPcmMenuItem );

    optMenu.add( optCpuMenuItem );
    optMenu.add( optVideoMenuItem );
    optMenu.add( optSoundMenu );
    optMenu.add( optInputMenuItem );
    
    // Tools
    toolMenu.add( toolPatternViewerMenuItem );

    // Help
    helpMenu.setMnemonic( KeyEvent.VK_H );
    helpMenu.add( helpAboutMenuItem );

    // create the actual menu bar
    mainMenu.add( fileMenu );
    mainMenu.add( cpuMenu );
    mainMenu.add( optMenu );
    mainMenu.add( toolMenu );
    mainMenu.add( helpMenu );
    // End Menu System --------------------------

    // Configure the FileChooser
    fileChooser = new JFileChooser();
    fileChooser.setMultiSelectionEnabled( false );
    fileChooser.addChoosableFileFilter( romTypeFilter );

    // Build the main view
    mainFrame.getContentPane().add( mainMenu,     BorderLayout.NORTH  );
    mainFrame.getContentPane().add( statusLabel,  BorderLayout.SOUTH );


    // ----------------------------------------------------------------------------
    // ----------------------------------------------------------------------------
    // ADD Listeners
    // ----------------------------------------------------------------------------
    // ----------------------------------------------------------------------------

    // Main window closing listener
    mainFrame.addWindowListener( new WindowAdapter()
    {
      public void windowClosing( WindowEvent we )
      {
        shutDown();
      }
    } );

    // File->Open menu listener
    fileOpenMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        openRom();
      }
    } );

    // File->Close menu listener
    fileCloseMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        closeRom();
      }
    } );

    // File->Exit menu listener
    fileExitMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        shutDown();
      }
    } );

    // CPU->Power menu listener
    cpuPowerMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        if( cpuPowerMenuItem.getState() )
        {
          startEmulation();
        }
      }
    } );

    // CPU->Reset menu listener
    cpuResetMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "CPU->Reset - Not Implemented" );
      }
    } );
    
    // Options->CPU... menu listener
    optCpuMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Options->CPU... - Not Implemented" );
      }
    } );

    // Options->Video... menu listener
    optVideoMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Options->Video... - Not Implemented" );
      }
    } );


    // Options->Sound->Square 1 menu listener
    optSndSquare1MenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Options->Sound->Square 1 - Not Implemented" );
      }
    } );

    // Options->Sound->Square 2 menu listener
    optSndSquare2MenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Options->Sound->Square 2 - Not Implemented" );
      }
    } );

    // Options->Sound->Triangle menu listener
    optSndTriangleMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Options->Sound->Triangle - Not Implemented" );
      }
    } );

    // Options->Sound->Noise menu listener
    optSndNoiseMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Options->Sound->Noise - Not Implemented" );
      }
    } );

    // Options->Sound->PCM menu listener
    optSndPcmMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Options->Sound->PCM - Not Implemented" );
      }
    } );


    // Options->Input... menu listener
    optInputMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Options->Input... - Not Implemented" );
      }
    } );
    
    // Tools->Pattern Table Viewer menu listener
    toolPatternViewerMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Tools->Pattern Table Viewer - Not Implemented" );
      }
    } );

    // Help->About menu listener
    helpAboutMenuItem.addActionListener( new ActionListener()
    {
      public void actionPerformed( ActionEvent ae )
      {
        System.out.println( "Help->About - Not Implemented" );
        helpAbout();
      }
    } );

  } // End of constructor ----------------------------------------------------------

  // Helper methods
  private void shutDown()
  {
    // For now just can everything...
    System.exit(0);
  }

  private void openRom()
  {
    if( rom != null )
      closeRom();
    
    fileChooser.setFileSelectionMode( JFileChooser.FILES_AND_DIRECTORIES );
    if( fileChooser.showOpenDialog( mainFrame ) == JFileChooser.APPROVE_OPTION )
    {
      rom = NesRomLoader.loadRomFile( fileChooser.getSelectedFile().getPath() );
      System.out.println( rom );
    }

    adjustMenus();
  }

  private void closeRom()
  {
    rom = null;
    statusLabel.setText( "Idle" );
    adjustMenus();
  }
  
  private void startEmulation()
  {
    MemoryMapperFactory mmf    = new MemoryMapperFactory( new NesMainMemory(), new NesVideoMemory(), rom );
    MemoryMapper        mapper = mmf.getMapper( rom.getInesRomHeader().getMapperNumber() );
    
    emulating = true;
    statusLabel.setText( "Emulating" );
    
    
    if( mapper != null )
    {
      System.out.println( mapper );
      mapper.powerOn();
    }
    else
    {
      statusLabel.setText( "Mapper Not Supported" );
    }  
  }  
  
  private void adjustMenus()
  {
    // If we are emulating, nothing related to options can be changed
    // If a rom is not loaded, emulation can not be started.

    if( rom == null )
    {
      fileMenu.setEnabled( true );
      fileOpenMenuItem.setEnabled( true );
      fileCloseMenuItem.setEnabled( true );
      fileExitMenuItem.setEnabled( true );
      cpuMenu.setEnabled( false );
      cpuPowerMenuItem.setEnabled( false );
      cpuPowerMenuItem.setState( false );            // State not enabled!
      cpuResetMenuItem.setEnabled( false );
      optMenu.setEnabled( true );
      optVideoMenuItem.setEnabled( true );
      optSoundMenu.setEnabled( true );
      optInputMenuItem.setEnabled( true );
      optSndSquare1MenuItem.setEnabled( true );
      optSndSquare2MenuItem.setEnabled( true );
      optSndTriangleMenuItem.setEnabled( true );
      optSndNoiseMenuItem.setEnabled( true );
      optSndPcmMenuItem.setEnabled( true );
      toolMenu.setEnabled( false );
      toolPatternViewerMenuItem.setEnabled( false );
      helpMenu.setEnabled( true );
      helpAboutMenuItem.setEnabled( true );
    }
    else
    {
      fileMenu.setEnabled( true );
      fileOpenMenuItem.setEnabled( true );
      fileCloseMenuItem.setEnabled( true );
      fileExitMenuItem.setEnabled( true );
      cpuMenu.setEnabled( true );
      cpuPowerMenuItem.setEnabled( true );
      cpuResetMenuItem.setEnabled( true );
      optMenu.setEnabled( false );
      optVideoMenuItem.setEnabled( false );
      optSoundMenu.setEnabled( false );
      optInputMenuItem.setEnabled( false );
      optSndSquare1MenuItem.setEnabled( false );
      optSndSquare2MenuItem.setEnabled( false );
      optSndTriangleMenuItem.setEnabled( false );
      optSndNoiseMenuItem.setEnabled( false );
      optSndPcmMenuItem.setEnabled( false );
      toolMenu.setEnabled( true );
      toolPatternViewerMenuItem.setEnabled( true );
      helpMenu.setEnabled( true );
      helpAboutMenuItem.setEnabled( true );
    }
  }

  private void helpAbout()
  {
    // helpAboutBox.show();
  }

  public void show()
  {
    // Position the window
    mainFrame.setLocation( new Point( 0, 0 ) );

    // Size the window
    mainFrame.setSize( 400, 400 );

    // Set the title
    mainFrame.setTitle( "MonkeyNES" );

    // Show the whole mess...
    mainFrame.setVisible( true );
    
    adjustMenus();
  }

}